package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Text; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.io.File;

/** action class for queries */
public class QueryAction extends Action {
    
    /** process - processes a request.
     */ 
    public Element process (Element message) {

	// get the request - assume there is only one
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);

	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	// add the system stuff from message
       page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

       // if want to have a different type of query here, check the subaction att of request

       // for now assume all queries can be handled by basic query
       return basicQuery(page, request);
       
    }
    
    /** a generic query handler 
     * this gets the service description, does the query (just passes all teh 
     * params to the service, then gets the titles for any results 
     */
    protected Element basicQuery(Element page, Element request) {

	// check that the stylesheet is present - cant output the page without one. 
	String stylesheet = GSFile.stylesheetFile(config_, "basicquery.xsl");
	if (stylesheet==null) {
	    System.err.println("QueryAction Error: basicquery stylesheet not found!");
	    return null;
	}

	// extract the params from the cgi-request, and check that we have a coll specified
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	
	HashMap params = GSXML.extractParams(cgi_param_list);
	
	String request_type = (String)params.get(GSCGI.REQUEST_TYPE_ARG);
	String service_name = (String)params.get(GSCGI.SERVICE_ARG);
	String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	if (collection == null || collection.equals("")) {
	    System.err.println("QueryAction Error: no collection was specified!");
	    return null; 
	}

	// get the service info from the MR - this will probably need to be cached somehow later on. and add the service node to the page 
	Element mr_info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_info_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_info_message.appendChild(mr_info_request);
	mr_info_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);
	mr_info_request.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));

	String to = collection;
	to = GSPath.appendLink(to, service_name);
	mr_info_request.setAttribute(GSXML.TO_ATT, to);

	Element mr_info_response = (Element) mr_.process(mr_info_message);
	String path = GSXML.RESPONSE_ELEM;
	path = GSPath.appendLink(path, GSXML.SERVICE_ELEM);
	Element description = (Element)doc_.importNode(GSXML.getNodeByPath(mr_info_response, path), true);
	Element pl = (Element)GSXML.getChildByTagName(description, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	
	if (pl !=null) {
	    System.out.println("adding shortnames");
	    // add short names to the params in the param list
	    cgi_.paramListAddShortNames(pl);
	    // for each param in the description, overwrite teh default value with the currently set value if present 
	    Element param = (Element)pl.getFirstChild();
	    while (param !=null) {
		if (param.getNodeName().equals(GSXML.PARAM_ELEM)) { // just in case
		    if (param.getAttribute(GSXML.TYPE_ATT).equals(GSXML.PARAM_TYPE_MULTI)) {
			// get the values for each sub param
			NodeList subparams = param.getElementsByTagName(GSXML.PARAM_ELEM);
			for (int i=0; i<subparams.getLength(); i++) {
			    String name = ((Element)subparams.item(i)).getAttribute(GSXML.NAME_ATT);
			    String current = (String)params.get(name);
			    if (current !=null && !current.equals("")) {
				Element e = GSXML.createTextElement(pl.getOwnerDocument(), GSXML.DEFAULT_ELEM, current);
				e.setAttribute(GSXML.NAME_ATT, name);
				param.appendChild(e);
			    }
			}
		    } else {
			
			String name = param.getAttribute(GSXML.NAME_ATT);
			String current = (String)params.get(name);
			if (current !=null && !current.equals("")) {
			    param.setAttribute(GSXML.DEFAULT_ATT, current);
			}
		    }
		}
		param = (Element)param.getNextSibling();
	    }
	}
    
	// part of the data for the description is the cgi-params
	// if we have this here, do we need to do the previous step?
	Element cgi_request = (Element)doc_.importNode(request, true);
	page.appendChild(cgi_request);

	page.appendChild(description);

	// just a display request
	if (request_type.equals("d")) {
	    // output the page
	    // process using the stylesheet
	    Document style_doc = converter_.getDOM(new File(stylesheet));
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return (Element)transformer_.transform(style_doc, page);    
	}
	    
	// do the query
	Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_query_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_query_message.appendChild(mr_query_request);
	
	mr_query_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	mr_query_request.setAttribute(GSXML.TO_ATT, to);
	mr_query_request.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	// paramList
	Element query_param_list = (Element)doc_.importNode(cgi_param_list, true);
	mr_query_request.appendChild(query_param_list);
	    
	    
	Element mr_query_response = (Element)mr_.process(mr_query_message);
	// System.out.println("Query response: " + converter_.getString(mr_query_response));

	Element query_result_metadata_list = (Element) GSXML.getNodeByPath(mr_query_response, "response/metadataList");
	if (query_result_metadata_list == null) {
	    System.err.println("Warning: No query result metadata.\n");
	}

	Element query_term_info_list = (Element) GSXML.getNodeByPath(mr_query_response, "response/content/termList");
	if (query_term_info_list == null) {
	    System.err.println("Warning: No query term information.\n");
	}

	// this result is the list of docs.
	// now take that list, and get the Titles
	// for now, just create a whole new request
	
	// check that there are some documents - for now check the list, but later should use a numdocs metadata elem
	path = GSXML.RESPONSE_ELEM;
	path = GSPath.appendLink(path, GSXML.CONTENT_ELEM);
	path = GSPath.appendLink(path, GSXML.DOCUMENT_ELEM+GSXML.LIST_MODIFIER);
	
	Element document_list = (Element)GSXML.getNodeByPath(mr_query_response,
							     path); // documentList not present if no docs found
	if (document_list == null) {
	    
	    Element result_response = (Element)GSXML.getChildByTagName(mr_query_response, GSXML.RESPONSE_ELEM);
	    
	    page.appendChild(doc_.importNode(result_response, true));
	    
	    Document style_doc = converter_.getDOM(new File(stylesheet));
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return (Element)transformer_.transform(style_doc, page);    
	    
	}
	
	// we have a doc list, so get the metadata - for now, get title.
	// can we dynamically decide what metadata to get?
	Element mr_metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_metadata_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_metadata_message.appendChild(mr_metadata_request);
	
	mr_metadata_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	mr_metadata_request.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	to = GSPath.appendLink(collection, "MetadataRetrieve");
	mr_metadata_request.setAttribute(GSXML.TO_ATT, to);
	
	Element meta_content = doc_.createElement(GSXML.CONTENT_ELEM);
	mr_metadata_request.appendChild(meta_content);
	
	// the first part of the content is the doc list
	meta_content.appendChild(doc_.importNode(document_list, true));
	
	// the second part of the content is the metadata list
	Element metadata_list = doc_.createElement(GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	Element title = doc_.createElement(GSXML.METADATA_ELEM);
	title.setAttribute(GSXML.NAME_ATT, "Title");
	metadata_list.appendChild(title);
	meta_content.appendChild(metadata_list);
	
	// System.out.println("Metadata message: " + converter_.getString(mr_metadata_message));
	Element mr_metadata_response = (Element)mr_.process(mr_metadata_message);
	
	Element result_response = (Element)GSXML.getChildByTagName(mr_metadata_response, GSXML.RESPONSE_ELEM);
	
	// If there is some metadata about the query results, add it in
	if (query_result_metadata_list != null) {
	    result_response.appendChild(query_result_metadata_list);
	}
	// If there is some metadata about the query results, add it in
	if (query_term_info_list != null) {
	    Element result_content = (Element) GSXML.getChildByTagName(result_response,
								       GSXML.CONTENT_ELEM);
	    result_content.appendChild(query_term_info_list);
	}

	page.appendChild(doc_.importNode(result_response, true));
	
	// System.out.println("XMLTransformer input: " + converter_.getString(page));
	
	// output the page
	// process using the stylesheet
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	// Node temp = transformer_.transform(style_doc, page);
        // System.out.println("XMLTransformer output: " + converter_.getString(temp));
	// return (Element) temp;
	return (Element)transformer_.transform(style_doc, page);
	
    }


}
